#!/bin/sh
# the next line restarts using tclsh \
exec tclsh "$0" "$@"

# TkfPW Tcl command

global env
set TkfPW_home $env(TKFPW_HOME)
set home $env(HOME)

source $TkfPW_home/lib/util.tcl
source $TkfPW_home/lib/pref.tcl

# Check for Arguments
if { [llength $argv] != 3 } {
	puts stderr "\n[file tail $argv0] :\n3 Arguments Needed !"
	exit
}

set suff [lindex $argv 0]

set DefOptFile [lindex $argv 1]
if ![ file exists $DefOptFile ] {
	puts stderr "[file tail $argv0] :\nInvalid File Pref : $DefOptFile"
	exit
}

set ListFiles [lindex $argv 2]
set nb_files [llength $ListFiles]
if { $nb_files == 0 } {
	puts stderr "[file tail $argv0] :\nNo Files to Compile !"
	exit
}

# Read in the DefOptFile
set Compiler [PrefTkfPW_read $DefOptFile {Compiler Name} {}]
set DefOptions [SetListValFlag01 [PrefTkfPW_read $DefOptFile {Compiler Options} {}]]

# Ok, I'm Ready to Compile ...
foreach file_abs $ListFiles {

	set file_f [file tail $file_abs]
	set pathfile_f [file dirname $file_abs]
	cd $pathfile_f

	regsub "\.$suff$" $file_f {} file
	set file_copt $pathfile_f/.$file.copt
	set file_errcp $pathfile_f/$file.errcp

	if [file exists $file_copt] {
		set list_copt [SetListValFlag01 [PrefTkfPW_read $file_copt {Compiler Options} ""]]
	} else {
		set list_copt $DefOptions
	}
	# get and join all active options together
	set list_copt_on [ join [GetListVal_FlagOn $list_copt] ]


	# If in options, change $HOME item to $home
	if [ string match *\$HOME* $list_copt_on ] {
		regsub -all {\$HOME} $list_copt_on $home list_copt_on
	}
	# If in options, change ~ item to $home
	if [ string match *~* $list_copt_on ] {
		regsub -all {~} $list_copt_on $home list_copt_on
	}


	puts "Compiling [file tail $file_f] ..."
	puts "Active Options :  $list_copt_on"

	catch {exec rm $file_errcp}

	if [catch {eval exec $Compiler -c $list_copt_on $file_abs} err] {
		puts "ERROR !"
		puts "$err"
		WriteVarInFile $err $file_errcp
	} else {
		puts "OK !\n"
	}

}

exit
