proc SetCompileOptions {title parent trace_type trace_id pathfiles files} {

	global Obj_Mes Obj_Dir Glob_Pref Process
	global SelOpt

	if ![file exists $Glob_Pref(File)] {
		tk_dialog .d {TkfPW Error in Editor} \
			{You have not set up the\nDefault Compilation Options\n\nPlease Configure TkfPW} \
			info 0 {Ooops !}
		return
	}

        # Create a toplevel window
	if ![winfo exists $parent] { set parent "" }
	set w $parent.lscp
	if [winfo exists $w] {
		wm deiconify $w
		return
	}

	toplevel $w
	wm title $w $title
    	wm transient $w [winfo toplevel [winfo parent $w]]
	wm protocol $w WM_DELETE_WINDOW { }
	#wm maxsize $w 270 400
	#wm minsize $w 270 300

	# Main Frame Container
	# ====================

	set wm $w.f
	frame $wm

	#################### Create a Frame General Infos
	set wi $wm.info
	frame $wi
	label $wi.title -text $title \
		-borderwidth 2 -relief ridge
	pack $wi.title -side top

	#################### Create a Frame General Buttons
	set wb $wm.buttons
	frame $wm.buttons -relief sunken -bd 1
	button $wb.done -text Done -command [list SetCompileOptions_Done $w $trace_type $trace_id ]
	pack $wb.done -side left -expand 1

	#################### Create a Frame for files selection
	set wf $wm.files
	frame $wf -bd 5 -relief ridge

	# Create the listbox in a frame
	set wf_lst $wf.fl
	frame $wf_lst -bd 2 -relief ridge
	set wlist [ScrolledListbox $wf_lst.main \
		-width 20 -height 10 -setgrid true]

	# Create the buttons for the listbox
	set wf_but $wf.fb
	frame $wf_but -bd 5 -relief ridge
	# this button is fully configured in the following ... when all widgets are defined
	button $wf_but.options -text "Options" -state disabled
	bind $wf_but.options  <Return> "$wf_but.options flash ; $wf_but.options invoke"
	button $wf_but.all -text "All" -command [list SetCompileOptions_ListSelectAll $wlist $wf_but.options]
	button $wf_but.none -text "None" -command [list SetCompileOptions_ListSelectNone $wlist $wf_but.options]
	pack $wf_but.all $wf_but.none $wf_but.options -side left

	# Create a Label for Dynamic info.
	set wf_infd $wf.infd
	frame $wf_infd -bd 5 -relief ridge
	label $wf_infd.status -textvariable Obj_Mes($wf.infd-status) \
		-width 30 -relief ridge
	pack $wf_infd.status -side top -padx 3 -pady 3 -expand yes

	# Create some static info for the listbox
	set wf_infs $wf.fi
	frame $wf_infs -bd 5 -relief ridge
	label $wf_infs.title -text "Files Selection" 
	pack $wf_infs.title -side top -fill x
	
	# Pack
	pack $wf_infs -side top -expand true -fill x
	pack $wf_lst -side top -expand true -fill both
	pack $wf_but $wf_infd -side top -pady 1m

	#################### Create a Frame for Options selection
	set wo $wm.options
	frame $wo -bd 5 -relief ridge

	# Create a Label for Static info.
	set wo_infs $wo.infs
	frame $wo_infs -bd 5 -relief ridge
	label $wo_infs.title -text "Options Selection"
	pack $wo_infs.title -side top -fill x

	# Create a Label for Dynamic info.
	set wo_infd $wo.infd
	frame $wo_infd -bd 5 -relief ridge
	label $wo_infd.status -textvariable Obj_Mes($wo_infd-status) \
		-width 30 -relief ridge
	pack $wo_infd.status -side top -padx 3 -pady 3 -expand yes

	# create frame for selection in canvas
	set wo_sel $wo.sel
	frame $wo_sel -bd 2 -relief ridge
		# Create a Scrollable Canvas for the Selection of Options!
		set wcanvas [ScrolledCanvas $wo_sel.cvs \
				-width 20 -height 10 ]
		# Create one frame to hold everything for the selection widgets
		# and position it on the canvas
		set wselopt [frame $wcanvas.wselopt]
		$wcanvas create window 0 0 -anchor nw -window $wselopt

	# Create the buttons for the options
	set wo_but $wo.button
	frame $wo_but  -bd 5 -relief ridge
	button $wo_but.add -text Add -cursor {plus} \
		-command [list SelOpt_AddOpt $wcanvas $wselopt $wo_infd-status]
	button $wo_but.undokill -text "Undo Kill" \
		-command [list SelOpt_UndoKillOpt $wcanvas $wselopt $wo_infd-status] -cursor {exchange}
	button $wo_but.save -text "Done" \
		-command [list SetCompileOptions_OptionsDone $wselopt $wo $wf_but $wo_but $wlist $pathfiles $wo_infd-status]
	button $wo_but.dismiss -text "Dismiss" \
		-command [list SetCompileOptions_OptionsDismiss $wo $wf_but $wo_but $wo_infd-status]
	menubutton $wo_but.reset -text {Reset} -menu $wo_but.reset.menu -relief raised
		set mreset [menu $wo_but.reset.menu -tearoff 0]
		# this menu is fully configured in the following ... when all widgets are defined
	pack $wo_but.dismiss $wo_but.save -side bottom -fill x
	pack $wo_but.add $wo_but.undokill $wo_but.reset -side left

	# Pack
	pack $wo_infs -side top -fill x
	pack $wo_infd $wo_but -side bottom -pady 1m
	pack $wo_sel -expand true -side top -fill both

	#################### Pack the Main Frames
	pack $wi -side top -expand true -fill both -padx 2m -pady 2m
	pack $wb -side bottom -expand true  -padx 2m -pady 2m
	pack $wf $wo -side left -expand true -fill both -padx 2m -pady 2m

	pack $wm -expand true -fill both

	center_window $w

	#################### Bindings the Files Buttons
	bind $wlist <ButtonPress-1> \
		[list SetCompileOptions_ListSelectStart %W %y $wf_but.options]
	bind $wlist <B1-Motion> \
		{SetCompileOptions_ListSelectExtend %W %y}
	bind $wlist <ButtonRelease-1> \
		[list SetCompileOptions_ListSelectEnd %W %y $wf_but.options]

	# Now, I am ready to configure the Activate Options Button and Reset Menu
	$wf_but.options configure \
		-command [list SetCompileOptions_ActivateOptions $wlist $wf_but $wo_but $pathfiles $wo $wcanvas $wselopt $wo.infd-status $mreset current]
	$mreset add command -label {Default Options} \
		-command [list SetCompileOptions_ActivateOptions $wlist $wf_but $wo_but $pathfiles $wo $wcanvas $wselopt $wo.infd-status $mreset default]
	$mreset add separator
	$mreset add command -label {Last Saved Options} \
		-command [list SetCompileOptions_ActivateOptions $wlist $wf_but $wo_but $pathfiles $wo $wcanvas $wselopt $wo.infd-status $mreset current]

	bind $wf  <Enter> [list ShowStatus "Ready ..." $wf.infd-status]
	bind $wf  <Leave> [list ShowStatus {} $wf.infd-status]
	bind $wf_but.all  <Enter> [list ShowStatus "Select All Files" $wf.infd-status]
	bind $wf_but.all  <Leave> [list ShowStatus "Ready ..." $wf.infd-status]
	bind $wf_but.none <Enter> [list ShowStatus "Unselect Selected Files" $wf.infd-status]
	bind $wf_but.none <Leave> [list ShowStatus "Ready ..." $wf.infd-status]
	bind $wf_but.options <Enter> [list ShowStatus "Activate Options Selection" $wf.infd-status]
	bind $wf_but.options <Leave> [list ShowStatus "Ready ..." $wf.infd-status]

	#################### Bindings the Options Buttons
	bind $wo  <Enter> [list ShowStatus "Ready ..." $wo.infd-status]
	bind $wo  <Leave> [list ShowStatus {} $wo.infd-status]
	bind $wo_but.add      <Enter> [list ShowStatus "Create a New Option" $wo.infd-status]
	bind $wo_but.add      <Leave> [list ShowStatus "Ready ..." $wo.infd-status]
	bind $wo_but.undokill <Enter> [list ShowStatus "Undo Last Killed" $wo.infd-status]
	bind $wo_but.undokill <Leave> [list ShowStatus "Ready ..." $wo.infd-status]
	bind $wo_but.save     <Enter> [list ShowStatus "Done : Save Options" $wo.infd-status]
	bind $wo_but.save     <Leave> [list ShowStatus "Ready ..." $wo.infd-status]
	bind $wo_but.dismiss  <Enter> [list ShowStatus "Done : Don't Save Options" $wo.infd-status]
	bind $wo_but.dismiss  <Leave> [list ShowStatus "Ready ..." $wo.infd-status]
	bind $wo_but.reset    <Enter> [list ShowStatus "Reset Options Menu" $wo.infd-status]
	bind $wo_but.reset    <Leave> [list ShowStatus "Ready ..." $wo.infd-status]

	# Update the Listbox ...
	foreach file $files {$wlist insert end $file}

	# Disable All in the Compiler Selection Options ...
	DisableButtonInW $wo_but

	# Ok, status is : running
	set Process($trace_type,selectopts,$trace_id) running

}
proc SetCompileOptions_Done {w trace_type trace_id} {
# End of selection
	global Process
	set Process($trace_type,selectopts,$trace_id) done
	catch {destroy $w}
}
proc SetCompileOptions_OptionsDone { wselopt wo wfbut wobut wlist path wstatus} {
global Glob_Pref
# End of selection : in the Option window (!)
	foreach i [$wlist curselection] {
		set file [$wlist get $i]
		ShowStatus "Saving Options for $file" $wstatus
		regsub "\.$Glob_Pref(FortranExt)$" $file {.copt} file_copt
		set file_copt $path/.$file_copt
		SetCompileOptions_save $wselopt $file_copt $wstatus
	}
	ShowStatus "Ready ..." $wstatus
	grab release $wo
	DisableButtonInW $wobut
	EnableButtonInW  $wfbut
}
proc SetCompileOptions_OptionsDismiss { wo wfbut wobut wstatus} {
	ShowStatus "Ready ..." $wstatus
	grab release $wo
	DisableButtonInW $wobut
	EnableButtonInW  $wfbut
}
proc SetCompileOptions_ListSelectStart { w y but} {
	$w select anchor [$w nearest $y]
	$but configure -state disabled
}
proc SetCompileOptions_ListSelectExtend { w y } {
	$w select set anchor [$w nearest $y]
}
proc SetCompileOptions_ListSelectAll {w option_button} {
	set num [llength [$w get 0 end] ]
	for {set i 0} {$i < $num} {incr i} {
		$w select set anchor $i
	}
	$option_button configure -state active
	focus $option_button
}
proc SetCompileOptions_ListSelectNone {w option_button} {
	$w selection clear 0 end
	$option_button configure -state disabled
}
proc SetCompileOptions_ListSelectEnd {w y option_button} {
	$w select set anchor [$w nearest $y]
	$option_button configure -state active
	focus $option_button
}
proc SetCompileOptions_ActivateOptions {wlist wfbut wobut path wo wcanvas wselopt wstatus mreset type} {
	global Glob_Pref

	# Clean the Option Window
	SelOpt_CleanWinOpt $wselopt

	# Update the Reset Menu
	$mreset entryconfigure 0 -state normal
	$mreset entryconfigure 2 -state normal

	# Generate a list of selected files
	set list_files {}
	set number_files 0
	foreach i [$wlist curselection] {
		incr number_files
		lappend list_files [$wlist get $i]
	}

	if { $number_files == 0 } {
		ShowStatus "No File(s) Selected !" $wstatus
		return
	}

	DisableButtonInW $wfbut
	EnableButtonInW  $wobut

	if { $number_files == 1 } {

	# 1 file to deal with

		set file [$wlist get $i]
		regsub "\.$Glob_Pref(FortranExt)$" $file {.copt} file_copt
		set file_copt $path/.$file_copt

		# try to read the" Option@Flag" from current...
		if [string match current $type] {
			set list_copt [SetListValFlag01 [PrefTkfPW_read $file_copt {Compiler Options} $wstatus]]
		} else {
			set list_copt {}
		}

		# if not, read the" Option@Flag" from default...
		if { $list_copt == {} } {
			set list_copt [SetListValFlag01 [PrefTkfPW_read $Glob_Pref(File) {Compiler Options} $wstatus]]
		}
		
	} else {

	# More than 1 file

		# Disable current reset
		$mreset entryconfigure 2 -state disabled

		# Read the" Option@Flag"
		set list_copt [SetListValFlag01 [PrefTkfPW_read $Glob_Pref(File) {Compiler Options} $wstatus]]
	}

	# Store the Option@Flag list into global array SelOpt
	SelOpt_ListToArray $list_copt $wselopt 

	# Update the Option Window and put a grab on it
	SelOpt_UpdateOpt $wcanvas $wselopt $wstatus
	grab $wo

}

proc SetCompileOptions_save {w file wstatus}  {
global SelOpt Glob_Pref
	# Open a stream for file to save
	if [catch {open $file w} fileId] {
		PRINT "proc SetCompileOptions_save : Cannot write $file : $fileId"
		tk_dialog .d {TkfPW Error} "Cannot write in file\n$file" info 0 Ok
		set list {}
	} else {
		# Write the Val@Flag items
		puts $fileId "# TkfPW Settings. Do not edit please."
		puts $fileId "Last Update @ [exec date "+%d/%m/%y %H:%M:%S"] @ $Glob_Pref(TKFPW_HOST) @ [exec uname -sr]"
		for {set  i 1} {$i <= $SelOpt($w,num)} {incr i} {
			puts $fileId "Compiler Options @ $SelOpt($w,$i,val)@$SelOpt($w,$i)"
		}
		close $fileId
	}
}
